function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
import { defineComponent } from '../utils/create';
import { easeInOutCubic, isPlainObject, toNumber } from '../utils/shared';
import { doubleRaf, getParentScroller, getScrollLeft, getScrollTop, nextTickFrame, requestAnimationFrame, scrollTo } from '../utils/elements';
import { props } from './props';
import { createParentMixin } from '../utils/mixins/relation';

var render = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    ref: "barEl",
    staticClass: "var-index-bar"
  }, [_vm._t("default"), _vm._v(" "), _c('ul', {
    staticClass: "var-index-bar__anchor-list",
    style: {
      zIndex: _vm.toNumber(_vm.zIndex) + 2,
      display: _vm.hideList ? 'none' : 'block'
    }
  }, _vm._l(_vm.anchorNameList, function (anchorName) {
    return _c('li', {
      key: anchorName,
      staticClass: "var-index-bar__anchor-item",
      class: {
        'var-index-bar__anchor-item--active': _vm.active === anchorName
      },
      style: {
        color: _vm.active === anchorName && _vm.highlightColor ? _vm.highlightColor : ''
      },
      on: {
        "click": function ($event) {
          return _vm.anchorClick(anchorName);
        }
      }
    }, [_vm._v("\n      " + _vm._s(anchorName) + "\n    ")]);
  }), 0)], 2);
};

var staticRenderFns = [];
export default defineComponent({
  render,
  staticRenderFns,
  name: 'VarIndexBar',
  mixins: [createParentMixin('indexBar', {
    childrenKey: 'indexAnchors'
  })],
  props,
  data: () => ({
    clickedName: '',
    scroller: null,
    anchorNameList: [],
    active: null
  }),

  mounted() {
    var _this = this;

    return _asyncToGenerator(function* () {
      yield doubleRaf();
      _this.scroller = getParentScroller(_this.$refs.barEl);

      _this.scroller.addEventListener('scroll', _this.handleScroll);
    })();
  },

  beforeDestroy() {
    var _this$scroller;

    (_this$scroller = this.scroller) == null ? void 0 : _this$scroller.removeEventListener('scroll', this.handleScroll);
  },

  watch: {
    indexAnchors(newValue) {
      var _this2 = this;

      return _asyncToGenerator(function* () {
        yield doubleRaf();
        newValue.forEach(_ref => {
          var {
            name,
            setOwnTop
          } = _ref;
          if (name) _this2.anchorNameList.push(name);
          setOwnTop();
        });
      })();
    }

  },
  methods: {
    // expose
    scrollTo(index) {
      requestAnimationFrame(() => this.anchorClick(index, true));
    },

    toNumber,

    emitEvent(anchor) {
      var _this$getListeners$on, _this$getListeners;

      var anchorName = isPlainObject(anchor) ? anchor.name : anchor;
      if (anchorName === this.active || anchorName === undefined) return;
      this.active = anchorName;
      (_this$getListeners$on = (_this$getListeners = this.getListeners()).onChange) == null ? void 0 : _this$getListeners$on.call(_this$getListeners, anchorName);
    },

    handleScroll() {
      var scrollTop = getScrollTop(this.scroller);
      var scrollHeight = this.scroller === window ? document.body.scrollHeight : this.scroller.scrollHeight;
      var {
        offsetTop
      } = this.$refs.barEl;
      this.indexAnchors.forEach((anchor, index) => {
        var anchorTop = anchor.ownTop;
        var top = scrollTop - anchorTop + this.stickyOffsetTop - offsetTop;
        var distance = index === this.indexAnchors.length - 1 ? scrollHeight : this.indexAnchors[index + 1].ownTop - anchor.ownTop;

        if (top >= 0 && top < distance && !this.clickedName) {
          if (index && !this.cssMode) {
            this.indexAnchors[index - 1].setDisabled(true);
          }

          anchor.setDisabled(false);
          this.emitEvent(anchor);
        }
      });
    },

    anchorClick(anchorName, manualCall) {
      var _this3 = this;

      return _asyncToGenerator(function* () {
        var _this3$getListeners$o, _this3$getListeners;

        if (manualCall) (_this3$getListeners$o = (_this3$getListeners = _this3.getListeners()).onClick) == null ? void 0 : _this3$getListeners$o.call(_this3$getListeners, anchorName);
        if (anchorName === _this3.active) return;

        var indexAnchor = _this3.indexAnchors.find(_ref2 => {
          var {
            name
          } = _ref2;
          return anchorName === name;
        });

        if (!indexAnchor) return;
        var top = indexAnchor.ownTop - _this3.stickyOffsetTop + _this3.$refs.barEl.offsetTop;
        var left = getScrollLeft(_this3.scroller);
        _this3.clickedName = anchorName;

        _this3.emitEvent(anchorName);

        yield scrollTo(_this3.scroller, {
          left,
          top,
          animation: easeInOutCubic,
          duration: toNumber(_this3.duration)
        });
        nextTickFrame(() => {
          _this3.clickedName = '';
        });
      })();
    }

  }
});