function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import context from '../context';


import { supportTouch } from '../utils/elements';
var ANIMATION_DURATION = 250;

function setStyles(element) {
  var {
    zIndex,
    position
  } = window.getComputedStyle(element);
  element.style.overflow = 'hidden';
  element.style.overflowX = 'hidden';
  element.style.overflowY = 'hidden';
  position === 'static' && (element.style.position = 'relative');
  zIndex === 'auto' && (element.style.zIndex = '1');
}

function computeRippleStyles(element, event) {
  var {
    top,
    left
  } = element.getBoundingClientRect();
  var {
    clientWidth,
    clientHeight
  } = element;
  var radius = Math.sqrt(Math.pow(clientWidth, 2) + Math.pow(clientHeight, 2)) / 2;
  var size = radius * 2;
  var localX = event.touches[0].clientX - left;
  var localY = event.touches[0].clientY - top;
  var centerX = (clientWidth - radius * 2) / 2;
  var centerY = (clientHeight - radius * 2) / 2;
  var x = localX - radius;
  var y = localY - radius;
  return {
    x,
    y,
    centerX,
    centerY,
    size
  };
}

function createRipple(event) {
  var _ripple = this._ripple;

  _ripple.removeRipple();

  if (_ripple.disabled || _ripple.tasker) {
    return;
  }

  var task = () => {
    var _ripple$color;

    _ripple.tasker = null;
    var {
      x,
      y,
      centerX,
      centerY,
      size
    } = computeRippleStyles(this, event);
    var ripple = document.createElement('div');
    ripple.classList.add('var-ripple');
    ripple.style.opacity = "0";
    ripple.style.transform = "translate(" + x + "px, " + y + "px) scale3d(.3, .3, .3)";
    ripple.style.width = size + "px";
    ripple.style.height = size + "px";
    ripple.style.backgroundColor = (_ripple$color = _ripple.color) != null ? _ripple$color : 'currentColor';
    ripple.dataset.createdAt = String(performance.now());
    setStyles(this);
    this.appendChild(ripple);
    window.setTimeout(() => {
      ripple.style.transform = "translate(" + centerX + "px, " + centerY + "px) scale3d(1, 1, 1)";
      ripple.style.opacity = ".25";
    }, 20);
  };

  _ripple.tasker = window.setTimeout(task, 60);
}

function removeRipple() {
  var _ripple = this._ripple;

  var task = () => {
    var ripples = this.querySelectorAll('.var-ripple');

    if (!ripples.length) {
      return;
    }

    var lastRipple = ripples[ripples.length - 1];
    var delay = ANIMATION_DURATION - performance.now() + Number(lastRipple.dataset.createdAt);
    setTimeout(() => {
      lastRipple.style.opacity = "0";
      setTimeout(() => {
        var _lastRipple$parentNod;

        return (_lastRipple$parentNod = lastRipple.parentNode) == null ? void 0 : _lastRipple$parentNod.removeChild(lastRipple);
      }, ANIMATION_DURATION);
    }, delay);
  };

  _ripple.tasker ? setTimeout(task, 60) : task();
}

function forbidRippleTask() {
  var _ripple = this._ripple;

  if (!supportTouch()) {
    return;
  }

  if (!_ripple.touchmoveForbid) {
    return;
  }

  _ripple.tasker && window.clearTimeout(_ripple.tasker);
  _ripple.tasker = null;
}

function inserted(el, binding) {
  var _binding$value, _binding$value$touchm, _binding$value2;

  el._ripple = _extends({
    tasker: null
  }, (_binding$value = binding.value) != null ? _binding$value : {}, {
    touchmoveForbid: (_binding$value$touchm = (_binding$value2 = binding.value) == null ? void 0 : _binding$value2.touchmoveForbid) != null ? _binding$value$touchm : context.touchmoveForbid,
    removeRipple: removeRipple.bind(el)
  });
  el.addEventListener('touchstart', createRipple, {
    passive: true
  });
  el.addEventListener('touchmove', forbidRippleTask, {
    passive: true
  });
  el.addEventListener('dragstart', removeRipple, {
    passive: true
  });
  document.addEventListener('touchend', el._ripple.removeRipple, {
    passive: true
  });
  document.addEventListener('touchcancel', el._ripple.removeRipple, {
    passive: true
  });
}

function unbind(el) {
  el.removeEventListener('touchstart', createRipple);
  el.removeEventListener('touchmove', forbidRippleTask);
  el.removeEventListener('dragstart', removeRipple);
  document.removeEventListener('touchend', el._ripple.removeRipple);
  document.removeEventListener('touchcancel', el._ripple.removeRipple);
}

function update(el, binding) {
  var _binding$value3, _binding$value$touchm2, _binding$value4;

  el._ripple = _extends({}, el._ripple, (_binding$value3 = binding.value) != null ? _binding$value3 : {}, {
    touchmoveForbid: (_binding$value$touchm2 = (_binding$value4 = binding.value) == null ? void 0 : _binding$value4.touchmoveForbid) != null ? _binding$value$touchm2 : context.touchmoveForbid,
    tasker: null
  });
}

var Ripple = {
  inserted,
  unbind,
  update,

  install(app) {
    app.directive('ripple', this);
  }

};
export var _RippleComponent = Ripple;
export default Ripple;