function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

import { isNumber, isString, kebabCase, toNumber } from './shared';
export function getLeft(element) {
  var {
    left
  } = element.getBoundingClientRect();
  return left + (document.body.scrollLeft || document.documentElement.scrollLeft);
}
export function getTop(element) {
  var {
    top
  } = element.getBoundingClientRect();
  return top + (document.body.scrollTop || document.documentElement.scrollTop);
}
export function getScrollTop(element) {
  var top = 'scrollTop' in element ? element.scrollTop : element.pageYOffset; // iOS scroll bounce cause minus scrollTop

  return Math.max(top, 0);
}
export function getScrollLeft(element) {
  var left = 'scrollLeft' in element ? element.scrollLeft : element.pageXOffset;
  return Math.max(left, 0);
}
export function doubleRaf() {
  return new Promise(resolve => {
    window.requestAnimationFrame(() => {
      window.requestAnimationFrame(resolve);
    });
  });
}
export function nextTickFrame(fn) {
  window.requestAnimationFrame(() => {
    window.requestAnimationFrame(fn);
  });
}
export function supportTouch() {
  var inBrowser = typeof window !== 'undefined';
  return inBrowser && 'ontouchstart' in window;
}
export function inViewport(_x) {
  return _inViewport.apply(this, arguments);
}

function _inViewport() {
  _inViewport = _asyncToGenerator(function* (element) {
    yield doubleRaf();
    var {
      top,
      bottom,
      left,
      right
    } = element.getBoundingClientRect();
    var {
      innerWidth,
      innerHeight
    } = window;
    var xInViewport = left <= innerWidth && right >= 0;
    var yInViewport = top <= innerHeight && bottom >= 0;
    return xInViewport && yInViewport;
  });
  return _inViewport.apply(this, arguments);
}

export function getTranslate(el) {
  var {
    transform
  } = window.getComputedStyle(el);
  return +transform.slice(transform.lastIndexOf(',') + 2, transform.length - 1);
}
export function getParentScroller(el) {
  var element = el;

  while (element) {
    if (!element.parentNode) {
      break;
    }

    element = element.parentNode;

    if (element === document.body || element === document.documentElement) {
      break;
    }

    var scrollRE = /(scroll|auto)/;
    var {
      overflowY,
      overflow
    } = window.getComputedStyle(element);

    if (scrollRE.test(overflowY) || scrollRE.test(overflow)) {
      return element;
    }
  }

  return window;
}
export function getAllParentScroller(el) {
  var allParentScroller = [];
  var element = el;

  while (element !== window) {
    element = getParentScroller(element);
    allParentScroller.push(element);
  }

  return allParentScroller;
} // example 1rem

export var isRem = value => isString(value) && value.endsWith('rem'); // example 1 || 1px

export var isPx = value => isString(value) && value.endsWith('px') || isNumber(value); // example 1%

export var isPercent = value => isString(value) && value.endsWith('%'); // example 1vw

export var isVw = value => isString(value) && value.endsWith('vw'); // example 1vh

export var isVh = value => isString(value) && value.endsWith('vh'); // example return 1

export var toPxNum = value => {
  if (isNumber(value)) {
    return value;
  }

  if (isPx(value)) {
    return +value.replace('px', '');
  }

  if (isVw(value)) {
    return +value.replace('vw', '') * window.innerWidth / 100;
  }

  if (isVh(value)) {
    return +value.replace('vh', '') * window.innerHeight / 100;
  }

  if (isRem(value)) {
    var num = +value.replace('rem', '');
    var rootFontSize = window.getComputedStyle(document.documentElement).fontSize;
    return num * parseFloat(rootFontSize);
  }

  if (isString(value)) {
    return toNumber(value);
  } // % and other


  return 0;
}; // example return 1px 1% 1vw 1vh 1rem null

export var toSizeUnit = value => {
  if (value == null) {
    return undefined;
  }

  if (isPercent(value) || isVw(value) || isVh(value) || isRem(value)) {
    return value;
  }

  return toPxNum(value) + "px";
};
export function requestAnimationFrame(fn) {
  return globalThis.requestAnimationFrame ? globalThis.requestAnimationFrame(fn) : globalThis.setTimeout(fn, 16);
}
export function cancelAnimationFrame(handle) {
  globalThis.cancelAnimationFrame ? globalThis.cancelAnimationFrame(handle) : globalThis.clearTimeout(handle);
}
export function scrollTo(element, _ref) {
  var {
    top = 0,
    left = 0,
    duration = 300,
    animation
  } = _ref;
  var startTime = Date.now();
  var scrollTop = getScrollTop(element);
  var scrollLeft = getScrollLeft(element);
  return new Promise(resolve => {
    var frame = () => {
      var progress = (Date.now() - startTime) / duration;

      if (progress < 1) {
        var nextTop = scrollTop + (top - scrollTop) * animation(progress);
        var nextLeft = scrollLeft + (left - scrollLeft) * animation(progress);
        element.scrollTo(nextLeft, nextTop);
        requestAnimationFrame(frame);
      } else {
        element.scrollTo(left, top);
        resolve();
      }
    };

    requestAnimationFrame(frame);
  });
}
export function isMobile() {
  return /Android|webOS|iPhone|iPod|BlackBerry|Pad/i.test(navigator.userAgent);
}
export function formatStyleVars(styleVars) {
  return Object.entries(styleVars != null ? styleVars : {}).reduce((styles, _ref2) => {
    var [key, value] = _ref2;
    var cssVar = key.startsWith('--') ? key : "--" + kebabCase(key);
    styles[cssVar] = value;
    return styles;
  }, {});
}