"use strict";

exports.__esModule = true;
exports.addLock = addLock;
exports.createLockMixin = createLockMixin;
exports.releaseLock = releaseLock;
exports.resolveLock = resolveLock;

var _vue = _interopRequireDefault(require("vue"));

var _ = _interopRequireDefault(require("."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function resolveLock() {
  var lockCounts = Object.keys(_.default.locks).length;
  lockCounts <= 0 ? document.body.classList.remove('m--lock') : document.body.classList.add('m--lock');
}

function addLock(uid) {
  _vue.default.set(_.default.locks, uid, 1);

  resolveLock();
}

function releaseLock(uid) {
  delete _.default.locks[uid];
  resolveLock();
}

function createLockMixin(state, use) {
  var propWatcher = use ? {
    [use](newValue) {
      if (!newValue) {
        releaseLock(this._uid);
      } else if (newValue && this[state]) {
        addLock(this._uid);
      }
    }

  } : {};
  return {
    watch: _extends({}, propWatcher, {
      [state](newValue) {
        if (use && !this[use]) {
          return;
        }

        if (newValue) {
          addLock(this._uid);
        } else {
          releaseLock(this._uid);
        }
      }

    }),

    beforeMount() {
      if (use && !this[use]) {
        return;
      }

      if (this[state]) {
        addLock(this._uid);
      }
    },

    beforeDestroy() {
      if (use && !this[use]) {
        return;
      }

      if (this[state]) {
        releaseLock(this._uid);
      }
    },

    activated() {
      if (use && !this[use]) {
        return;
      }

      if (this[state]) {
        addLock(this._uid);
      }
    },

    deactivated() {
      if (use && !this[use]) {
        return;
      }

      if (this[state]) {
        releaseLock(this._uid);
      }
    }

  };
}