"use strict";

exports.__esModule = true;
exports.default = void 0;

var _create = require("../../utils/create");

var _dayjs = _interopRequireDefault(require("dayjs"));

var _isSameOrBefore = _interopRequireDefault(require("dayjs/plugin/isSameOrBefore"));

var _isSameOrAfter = _interopRequireDefault(require("dayjs/plugin/isSameOrAfter"));

var _props = require("../props");

var _panelHeader = _interopRequireDefault(require("./panel-header.js"));

var _button = _interopRequireDefault(require("../../button"));

var _shared = require("../../utils/shared");

var _locale = require("../../locale");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
_dayjs.default.extend(_isSameOrBefore.default);

_dayjs.default.extend(_isSameOrAfter.default);

var render = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    staticClass: "var-month-picker__panel"
  }, [_c('div', {
    staticClass: "var-month-picker__content"
  }, [_c('panel-header', {
    ref: "headerEl",
    attrs: {
      "type": "month",
      "date": _vm.preview,
      "disabled": _vm.panelBtnDisabled
    },
    on: {
      "check-panel": _vm.clickYear,
      "check-date": _vm.checkDate
    }
  }), _vm._v(" "), _c('transition', {
    attrs: {
      "name": "var-date-picker" + (_vm.reverse ? '-reverse' : '') + "-translatex"
    }
  }, [_c('ul', {
    key: _vm.panelKey
  }, _vm._l(_vm.MONTH_LIST, function (month) {
    return _c('li', {
      key: month.index
    }, [_c('var-button', _vm._b({
      staticClass: "var-month-picker__button",
      attrs: {
        "type": "primary",
        "var-month-picker-cover": "",
        "ripple": false
      },
      on: {
        "click": function (event) {
          return _vm.chooseMonth(month, event);
        }
      }
    }, 'var-button', Object.assign({}, _vm.buttonProps(month.index)), false), [_vm._v("\n            " + _vm._s(_vm.getMonthAbbr(month.index)) + "\n          ")])], 1);
  }), 0)])], 1)]);
};

var staticRenderFns = [];

var _default = (0, _create.defineComponent)({
  render,
  staticRenderFns,
  name: 'MonthPickerPanel',
  components: {
    VarButton: _button.default,
    PanelHeader: _panelHeader.default
  },
  props: {
    choose: {
      type: Object,
      required: true
    },
    preview: {
      type: Object,
      required: true
    },
    current: {
      type: String,
      required: true
    },
    clickYear: {
      type: Function,
      required: true
    },
    componentProps: {
      type: Object,
      required: true
    }
  },
  data: () => ({
    reverse: false,
    panelKey: 0,
    panelBtnDisabled: {
      left: false,
      right: false
    }
  }),
  computed: {
    pack() {
      return _locale.pack.value;
    },

    MONTH_LIST() {
      return _props.MONTH_LIST;
    },

    currentYear() {
      return this.current.split('-')[0];
    },

    currentMonth() {
      return this.current.split('-')[1];
    },

    isSameYear() {
      return this.choose.chooseYear === this.preview.previewYear;
    },

    isCurrentYear() {
      return this.preview.previewYear === this.currentYear;
    }

  },
  watch: {
    'preview.previewYear': {
      handler(year) {
        var {
          componentProps: {
            min,
            max
          }
        } = this;
        if (max) this.panelBtnDisabled.right = !(0, _dayjs.default)("" + ((0, _shared.toNumber)(year) + 1)).isSameOrBefore((0, _dayjs.default)(max), 'year');
        if (min) this.panelBtnDisabled.left = !(0, _dayjs.default)("" + ((0, _shared.toNumber)(year) - 1)).isSameOrAfter((0, _dayjs.default)(min), 'year');
      },

      immediate: true
    }
  },
  methods: {
    getMonthAbbr(key) {
      var _this$pack$datePicker, _this$pack$datePicker2;

      return (_this$pack$datePicker = (_this$pack$datePicker2 = this.pack.datePickerMonthDict) == null ? void 0 : _this$pack$datePicker2[key].abbr) != null ? _this$pack$datePicker : '';
    },

    inRange(key) {
      var {
        preview: {
          previewYear
        },
        componentProps: {
          min,
          max
        }
      } = this;
      var isBeforeMax = true;
      var isAfterMin = true;
      var previewDate = previewYear + "-" + key;
      if (max) isBeforeMax = (0, _dayjs.default)(previewDate).isSameOrBefore((0, _dayjs.default)(max), 'month');
      if (min) isAfterMin = (0, _dayjs.default)(previewDate).isSameOrAfter((0, _dayjs.default)(min), 'month');
      return isBeforeMax && isAfterMin;
    },

    shouldChoose(val) {
      var {
        choose: {
          chooseMonths,
          chooseDays,
          chooseRangeMonth
        },
        componentProps: {
          type,
          range
        }
      } = this;

      if (range) {
        if (!chooseRangeMonth.length) return false;
        var isBeforeMax = (0, _dayjs.default)(val).isSameOrBefore((0, _dayjs.default)(chooseRangeMonth[1]), 'month');
        var isAfterMin = (0, _dayjs.default)(val).isSameOrAfter((0, _dayjs.default)(chooseRangeMonth[0]), 'month');
        return isBeforeMax && isAfterMin;
      }

      if (type === 'month') return chooseMonths.includes(val);
      return chooseDays.some(value => value.includes(val));
    },

    buttonProps(key) {
      var {
        choose: {
          chooseMonth
        },
        preview: {
          previewYear
        },
        componentProps: {
          allowedDates,
          color,
          multiple,
          range
        },
        shouldChoose,
        isSameYear,
        inRange,
        isCurrentYear,
        currentMonth
      } = this;
      var val = previewYear + "-" + key;

      var monthExist = () => {
        if (range || multiple) return shouldChoose(val);
        return (chooseMonth == null ? void 0 : chooseMonth.index) === key && isSameYear;
      };

      var computeDisabled = () => {
        if (!inRange(key)) return true;
        if (!allowedDates) return false;
        return !allowedDates(val);
      };

      var disabled = computeDisabled();

      var computeText = () => {
        if (disabled) return true;
        if (range || multiple) return !shouldChoose(val);
        return !isSameYear || (chooseMonth == null ? void 0 : chooseMonth.index) !== key;
      };

      var computeOutline = () => {
        // 不满足基本条件， 基本条件为当前年、当前月并且 showCurrent 为true的情况
        if (!(isCurrentYear && currentMonth === key && this.componentProps.showCurrent)) return false; // 存在着 disabled

        if ((range || multiple || isSameYear) && disabled) return true; // 在选择范围之外

        if (range || multiple) return !shouldChoose(val); // 同一年但是未被选择的情况

        if (isSameYear) return (chooseMonth == null ? void 0 : chooseMonth.index) !== currentMonth;
        return true;
      };

      var textColorOrCover = () => {
        if (disabled) return '';
        if (computeOutline()) return color != null ? color : '';
        if (monthExist()) return '';
        return 'var-date-picker-color-cover';
      };

      var isCover = textColorOrCover().startsWith('var-date-picker');
      return {
        disabled,
        outline: computeOutline(),
        text: computeText(),
        color: !computeText() ? color : '',
        textColor: isCover ? '' : textColorOrCover(),
        'var-date-picker-color-cover': isCover,
        class: {
          'var-month-picker__button-disabled': disabled
        }
      };
    },

    chooseMonth(month, event) {
      var buttonEl = event.currentTarget;
      if (buttonEl.classList.contains('var-month-picker__button-disabled')) return;
      this.$emit('choose-month', month);
    },

    checkDate(checkType) {
      this.reverse = checkType === 'prev';
      this.panelKey += checkType === 'prev' ? -1 : 1;
      this.$emit('check-preview', 'year', checkType);
    },

    forwardRef(checkType) {
      this.$refs.headerEl.checkDate(checkType);
    }

  }
});

exports.default = _default;