"use strict";

exports.__esModule = true;
exports.default = exports._DialogComponent = void 0;

var _Dialog = _interopRequireDefault(require("./Dialog.js"));

var _vue = _interopRequireDefault(require("vue"));

var _shared = require("../utils/shared");

var _components = require("../utils/components");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var singletonInstance;

function Dialog(options) {
  if (!(0, _shared.inBrowser)()) {
    return Promise.resolve();
  }

  return new Promise(resolve => {
    Dialog.close();
    var dialogOptions = (0, _shared.isString)(options) || (0, _shared.isNumber)(options) ? {
      message: String(options)
    } : options;
    var {
      instance,
      unmount
    } = (0, _components.mountComponent)(_Dialog.default, 'body', {
      propsData: _extends({}, dialogOptions, {
        beforeClose: dialogOptions.onBeforeClose
      })
    });
    instance.show = true;
    singletonInstance = instance;
    instance.$on('open', () => dialogOptions.onOpen == null ? void 0 : dialogOptions.onOpen());
    instance.$on('opened', () => dialogOptions.onOpened == null ? void 0 : dialogOptions.onOpened());
    instance.$on('click-overlay', () => dialogOptions.onClickOverlay == null ? void 0 : dialogOptions.onClickOverlay());
    instance.$on('close', () => {
      dialogOptions.onClose == null ? void 0 : dialogOptions.onClose();
      resolve('close');
    });
    instance.$on('closed', () => {
      dialogOptions.onClosed == null ? void 0 : dialogOptions.onClosed();
      unmount();
      singletonInstance === instance && (singletonInstance = null);
    });
    instance.$on('confirm', () => {
      dialogOptions.onConfirm == null ? void 0 : dialogOptions.onConfirm();
      resolve('confirm');
    });
    instance.$on('cancel', () => {
      dialogOptions.onCancel == null ? void 0 : dialogOptions.onCancel();
      resolve('cancel');
    });
    instance.$on('route-change', () => {
      unmount();
      singletonInstance === instance && (singletonInstance = null);
    });
    instance.$on('update:show', value => {
      instance.show = value;
    });
  });
}

Dialog.close = () => {
  if (singletonInstance != null) {
    var prevSingletonInstance = singletonInstance;
    singletonInstance = null;

    _vue.default.nextTick(() => {
      prevSingletonInstance.show = false;
    });
  }
};

_Dialog.default.install = function (app) {
  app.component(_Dialog.default.name, _Dialog.default);
};

Dialog.install = function (app) {
  app.component(_Dialog.default.name, _Dialog.default);
};

Dialog.Component = _Dialog.default;
var _DialogComponent = _Dialog.default;
exports._DialogComponent = _DialogComponent;
var _default = Dialog;
exports.default = _default;