"use strict";

exports.__esModule = true;
exports.default = void 0;

var _create = require("../utils/create");

var _props = require("./props");

var _teleport = _interopRequireDefault(require("../teleport"));

var _elements = require("../utils/elements");

var _zIndex = require("../utils/mixins/zIndex");

var _teleport2 = require("../utils/mixins/teleport");

var _shared = require("../utils/shared");







function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var _default = (0, _create.defineComponent)({
  name: 'VarMenu',
  mixins: [(0, _zIndex.createZIndexMixin)('show', 1), _teleport2.createTeleportMixin],
  props: _props.props,
  data: () => ({
    to: null,
    top: 0,
    left: 0,
    clickSelf: false
  }),
  computed: {
    transitionStyle() {
      return {
        top: "calc(" + this.top + "px + " + (0, _elements.toSizeUnit)(this.offsetY) + ")",
        left: "calc(" + this.left + "px + " + (0, _elements.toSizeUnit)(this.offsetX) + ")",
        zIndex: this.zIndex
      };
    }

  },

  mounted() {
    this.to = this.teleport;
    this.resize();
    document.addEventListener('click', this.handleMenuClose);
    window.addEventListener('resize', this.resize);
  },

  beforeDestroy() {
    document.removeEventListener('click', this.handleMenuClose);
    window.removeEventListener('resize', this.resize);
  },

  watch: {
    alignment() {
      this.resize();
    },

    show(newValue) {
      var _this = this;

      return _asyncToGenerator(function* () {
        var {
          onOpen,
          onClose
        } = _this.getListeners();

        if (newValue) {
          yield (0, _create.nextTick)();

          _this.resize();
        }

        newValue ? onOpen == null ? void 0 : onOpen() : onClose == null ? void 0 : onClose();
      })();
    },

    teleport(newValue) {
      this.to = newValue;
    }

  },
  methods: {
    // expose
    resize() {
      var {
        host
      } = this.$refs;
      this.top = this.computeTop(this.alignment);

      if (!host) {
        return;
      }

      this.left = (0, _elements.getLeft)(host);
    },

    computeTop(alignment) {
      var {
        host,
        menu
      } = this.$refs;

      if (!host) {
        return;
      }

      return alignment === 'top' ? (0, _elements.getTop)(host) : (0, _elements.getTop)(host) - menu.offsetHeight;
    },

    handleClick() {
      this.clickSelf = true;
    },

    handleMenuClose() {
      var _this$getListeners$on, _this$getListeners;

      if (this.clickSelf) {
        this.clickSelf = false;
        return;
      }

      if (!this.show) {
        return;
      }

      (_this$getListeners$on = (_this$getListeners = this.getListeners())['onUpdate:show']) == null ? void 0 : _this$getListeners$on.call(_this$getListeners);
    },

    renderTransition() {
      var _this2 = this;

      var h = this.$createElement;
      return h("transition", {
        "attrs": {
          "name": "var-menu"
        },
        "on": _extends({}, {
          "afterEnter": function ($event) {
            var _this2$getListeners$o;

            for (var _len = arguments.length, attrs = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
              attrs[_key - 1] = arguments[_key];
            }

            ((_this2$getListeners$o = _this2.getListeners().onOpened) != null ? _this2$getListeners$o : _shared.NOOP)($event, ...attrs);
          },
          "afterLeave": function ($event) {
            var _this2$getListeners$o2;

            for (var _len2 = arguments.length, attrs = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
              attrs[_key2 - 1] = arguments[_key2];
            }

            ((_this2$getListeners$o2 = _this2.getListeners().onClosed) != null ? _this2$getListeners$o2 : _shared.NOOP)($event, ...attrs);
          }
        })
      }, [h("div", {
        "class": "var-menu__menu var-elevation--3",
        "ref": "menu",
        "style": this.transitionStyle,
        "directives": [{
          name: "show",
          value: this.show
        }],
        "on": _extends({}, {
          "click": function ($event) {
            for (var _len3 = arguments.length, attrs = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
              attrs[_key3 - 1] = arguments[_key3];
            }

            (event => {
              event.stopPropagation();
            })($event, ...attrs);
          }
        })
      }, [this.slots('menu')])]);
    }

  },

  render() {
    var _this3 = this;

    var h = arguments[0];
    return h("div", {
      "class": "var-menu",
      "ref": "host",
      "on": _extends({}, {
        "click": function ($event) {
          for (var _len4 = arguments.length, attrs = new Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
            attrs[_key4 - 1] = arguments[_key4];
          }

          _this3.handleClick($event, ...attrs);
        }
      })
    }, [this.slots('default'), this.to ? h(_teleport.default, {
      "attrs": {
        "to": this.to,
        "disabled": this.teleportDisabled
      }
    }, [this.renderTransition()]) : this.renderTransition()]);
  }

});

exports.default = _default;