"use strict";

exports.__esModule = true;
exports.default = void 0;

var _icon = _interopRequireDefault(require("../icon"));

var _create = require("../utils/create");

var _elements = require("../utils/elements");

var _props = require("./props");

var _shared = require("../utils/shared");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
var MAX_DISTANCE = 100;
var CONTROL_POSITION = -50;
var scroller;

var render = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    ref: "freshNode",
    staticClass: "var-pull-refresh",
    on: {
      "touchstart": _vm.touchStart,
      "touchmove": _vm.touchMove,
      "touchend": _vm.touchEnd,
      "touchcancel": _vm.touchEnd
    }
  }, [_c('div', {
    staticClass: "var-pull-refresh__control var-elevation--2",
    class: [_vm.isSuccess ? 'var-pull-refresh__control-success' : null],
    style: _vm.controlStyle
  }, [_c('var-icon', {
    class: _vm.iconClass,
    attrs: {
      "name": _vm.iconName,
      "transition": 200,
      "var-pull-refresh-cover": ""
    }
  })], 1), _vm._v(" "), _vm._t("default")], 2);
};

var staticRenderFns = [];

var _default = (0, _create.defineComponent)({
  render,
  staticRenderFns,
  name: 'VarPullRefresh',
  components: {
    VarIcon: _icon.default
  },
  props: _props.props,
  data: () => ({
    freshNode: null,
    startPosition: 0,
    distance: CONTROL_POSITION,
    iconName: 'arrow-down',
    refreshStatus: 'default',
    isEnd: false
  }),
  computed: {
    isTouchable() {
      return this.refreshStatus !== 'loading' && this.refreshStatus !== 'success' && !this.disabled;
    },

    iconClass() {
      return {
        'var-pull-refresh__icon': true,
        'var-pull-refresh__animation': this.refreshStatus === 'loading'
      };
    },

    controlStyle() {
      return {
        transform: "translate3d(0px, " + this.distance + "px, 0px) translate(-50%, 0)",
        transition: this.isEnd ? "transform " + this.animationDuration + "ms" : undefined,
        background: this.successBgColor || this.bgColor,
        color: this.successColor || this.color
      };
    },

    isSuccess() {
      return this.refreshStatus === 'success';
    }

  },

  mounted() {
    scroller = (0, _elements.getParentScroller)(this.freshNode);
  },

  watch: {
    value(newValue) {
      if (newValue === false) {
        this.isEnd = true;
        this.refreshStatus = 'success';
        this.iconName = 'checkbox-marked-circle';
        setTimeout(() => {
          this.distance = CONTROL_POSITION;
          this.reset();
        }, (0, _shared.toNumber)(this.successDuration));
      }
    }

  },
  methods: {
    touchStart(event) {
      if (!this.isTouchable) return;
      this.refreshStatus = 'pulling';
      this.startPosition = event.touches[0].clientY;
    },

    touchMove(event) {
      var scrollTop = (0, _elements.getScrollTop)(scroller);
      if (scrollTop > 0 || !this.isTouchable) return;
      if (scrollTop === 0 && this.distance > CONTROL_POSITION) event.cancelable && event.preventDefault();
      var moveDistance = (event.touches[0].clientY - this.startPosition) / 2 + CONTROL_POSITION;
      this.distance = moveDistance >= MAX_DISTANCE ? MAX_DISTANCE : moveDistance;
      this.iconName = this.distance >= MAX_DISTANCE * 0.2 ? 'refresh' : 'arrow-down';
    },

    touchEnd() {
      if (!this.isTouchable) return;
      this.isEnd = true;

      if (this.distance >= MAX_DISTANCE * 0.2) {
        var _this$getListeners$on, _this$getListeners, _this$getListeners$on2, _this$getListeners2;

        this.refreshStatus = 'loading';
        this.distance = MAX_DISTANCE * 0.3;
        (_this$getListeners$on = (_this$getListeners = this.getListeners()).onInput) == null ? void 0 : _this$getListeners$on.call(_this$getListeners, true);
        (_this$getListeners$on2 = (_this$getListeners2 = this.getListeners()).onRefresh) == null ? void 0 : _this$getListeners$on2.call(_this$getListeners2);
      } else {
        this.refreshStatus = 'loosing';
        this.iconName = 'arrow-down';
        this.distance = CONTROL_POSITION;
        setTimeout(() => {
          this.isEnd = false;
        }, (0, _shared.toNumber)(this.animationDuration));
      }
    },

    reset() {
      setTimeout(() => {
        this.refreshStatus = 'default';
        this.iconName = 'arrow-down';
        this.isEnd = false;
      }, (0, _shared.toNumber)(this.animationDuration));
    }

  }
});

exports.default = _default;