"use strict";

exports.__esModule = true;
exports.default = exports._SnackbarComponent = exports.SNACKBAR_TYPE = void 0;

var _vue = _interopRequireDefault(require("vue"));

var _core = _interopRequireDefault(require("./core.js"));

var _Snackbar = _interopRequireDefault(require("./Snackbar.js"));

var _context = _interopRequireDefault(require("../context"));

var _components = require("../utils/components");

var _shared = require("../utils/shared");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var SNACKBAR_TYPE = ['loading', 'success', 'warning', 'info', 'error'];
exports.SNACKBAR_TYPE = SNACKBAR_TYPE;
var sid = 0;
var isMount = false;
var unmount;
var isAllowMultiple = false;

var uniqSnackbarOptions = _vue.default.observable({
  value: []
});

var defaultOption = {
  type: undefined,
  content: '',
  position: 'top',
  duration: 3000,
  vertical: false,
  contentClass: undefined,
  loadingType: 'circle',
  loadingSize: 'normal',
  lockScroll: false,
  teleport: 'body',
  forbidClick: false,
  onOpen: () => {},
  onOpened: () => {},
  onClose: () => {},
  onClosed: () => {}
};
var transitionGroupProps = {
  name: 'var-snackbar-fade',
  tag: 'div',
  class: 'var-transition-group'
};

function getCoreVNode(h, option) {
  var {
    id,
    reactiveSnackOptions,
    customUpdate
  } = option;

  var attrs = _extends({}, reactiveSnackOptions, {
    'data-id': id,
    customUpdate
  });

  var directives = [{
    name: 'show',
    rawName: 'v-show',
    value: reactiveSnackOptions.show,
    expression: 'show'
  }];
  var style = {
    position: isAllowMultiple ? 'relative' : 'absolute',
    top: getTop(reactiveSnackOptions.position)
  };
  var on = {
    'update:show': value => {
      reactiveSnackOptions.show = value;
    },
    open: () => {
      reactiveSnackOptions == null ? void 0 : reactiveSnackOptions.onOpen();
    },
    close: () => {
      reactiveSnackOptions == null ? void 0 : reactiveSnackOptions.onClose();
    }
  };
  return h(_core.default, {
    key: id,
    attrs,
    style,
    directives,
    on
  });
}

var TransitionGroupHost = {
  render(h) {
    var snackbarList = uniqSnackbarOptions.value.map(_ref => {
      var {
        id,
        reactiveSnackOptions,
        customUpdate
      } = _ref;
      var transitionGroupEl = document.querySelector('.var-transition-group');

      if (reactiveSnackOptions.forbidClick || reactiveSnackOptions.type === 'loading') {
        transitionGroupEl.classList.add('var-pointer-auto');
      } else {
        transitionGroupEl.classList.remove('var-pointer-auto');
      }

      if (isAllowMultiple) reactiveSnackOptions.position = 'top';
      return getCoreVNode(h, {
        id,
        reactiveSnackOptions,
        customUpdate
      });
    });
    return h('transition-group', {
      attrs: _extends({}, transitionGroupProps),
      style: {
        zIndex: _context.default.zIndex
      },
      on: {
        'after-enter': opened,
        'after-leave': removeUniqOption
      }
    }, snackbarList);
  }

};

var Snackbar = function (options) {
  var snackOptions = (0, _shared.isString)(options) || (0, _shared.isNumber)(options) ? {
    content: String(options)
  } : options;

  var reactiveSnackOptions = _vue.default.observable(_extends({}, defaultOption, snackOptions));

  _vue.default.set(reactiveSnackOptions, 'show', true);

  if (!isMount) {
    isMount = true;
    unmount = (0, _components.mountComponent)(TransitionGroupHost).unmount;
  }

  var {
    length
  } = uniqSnackbarOptions.value;
  var uniqSnackbarOptionItem = {
    id: sid++,
    reactiveSnackOptions
  };

  if (length === 0 || isAllowMultiple) {
    addUniqOption(uniqSnackbarOptionItem);
  } else {
    var customUpdate = "update-" + sid;
    updateUniqOption(reactiveSnackOptions, customUpdate);
  }

  return {
    clear() {
      if (!isAllowMultiple && uniqSnackbarOptions.value.length) {
        uniqSnackbarOptions.value[0].reactiveSnackOptions.show = false;
      } else {
        reactiveSnackOptions.show = false;
      }
    }

  };
};

SNACKBAR_TYPE.forEach(type => {
  Snackbar[type] = options => {
    if ((0, _shared.isPlainObject)(options)) {
      options.type = type;
    } else {
      options = {
        content: options,
        type
      };
    }

    return Snackbar(options);
  };
});

Snackbar.install = function (app) {
  app.component(_Snackbar.default.name, _Snackbar.default);
};

Snackbar.allowMultiple = function (bool) {
  if (bool === void 0) {
    bool = false;
  }

  if (bool !== isAllowMultiple) {
    uniqSnackbarOptions.value.forEach(option => {
      option.reactiveSnackOptions.show = false;
    });
    isAllowMultiple = bool;
  }
};

Snackbar.clear = function () {
  uniqSnackbarOptions.value.forEach(option => {
    option.reactiveSnackOptions.show = false;
  });
};

Snackbar.Component = _Snackbar.default;

function opened(element) {
  var id = element.getAttribute('data-id');
  var option = uniqSnackbarOptions.value.find(option => option.id === (0, _shared.toNumber)(id));
  if (option) option.reactiveSnackOptions.onOpened == null ? void 0 : option.reactiveSnackOptions.onOpened();
}

function removeUniqOption(element) {
  element.parentElement && element.parentElement.classList.remove('var-pointer-auto');
  var id = element.getAttribute('data-id');
  var option = uniqSnackbarOptions.value.find(option => option.id === (0, _shared.toNumber)(id));

  if (option) {
    option.animationEnd = true;
    option.reactiveSnackOptions.onClosed == null ? void 0 : option.reactiveSnackOptions.onClosed();
  }

  var isAllAnimationEnd = uniqSnackbarOptions.value.every(option => option.animationEnd);

  if (isAllAnimationEnd) {
    unmount == null ? void 0 : unmount();
    uniqSnackbarOptions = _vue.default.observable({
      value: []
    });
    isMount = false;
  }
}

function addUniqOption(uniqSnackbarOptionItem) {
  uniqSnackbarOptions.value.push(uniqSnackbarOptionItem);
}

function updateUniqOption(reactiveSnackOptions, customUpdate) {
  var [firstOption] = uniqSnackbarOptions.value;
  firstOption.reactiveSnackOptions = _extends({}, firstOption.reactiveSnackOptions, reactiveSnackOptions);
  firstOption.customUpdate = customUpdate;
}

function getTop(position) {
  if (position === void 0) {
    position = 'top';
  }

  if (position === 'center') return '45%';
  if (position === 'bottom') return '85%';
  return '50px';
}

_Snackbar.default.install = function (app) {
  app.component(_Snackbar.default.name, _Snackbar.default);
};

var _SnackbarComponent = _Snackbar.default;
exports._SnackbarComponent = _SnackbarComponent;
var _default = Snackbar;
exports.default = _default;