"use strict";

exports.__esModule = true;
exports.default = void 0;

var _create = require("../utils/create");

var _dayjs = _interopRequireDefault(require("dayjs"));

var _props = require("./props");

var _utils = require("./utils");

var _shared = require("../utils/shared");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
var render = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    staticClass: "var-time-picker-clock"
  }, [_c('div', {
    staticClass: "var-time-picker-clock__hand",
    style: _vm.handStyle
  }), _vm._v(" "), _vm._l(_vm.timeScales, function (timeScale, index) {
    return _c('div', {
      key: timeScale,
      staticClass: "var-time-picker-clock__item",
      class: [_vm.isActive(index, false) ? 'var-time-picker-clock__item--active' : null, _vm.isDisable(timeScale) ? 'var-time-picker-clock__item--disable' : null],
      style: _vm.getStyle(index, timeScale, false)
    }, [_vm._v("\n    " + _vm._s(timeScale) + "\n  ")]);
  }), _vm._v(" "), _vm.format === '24hr' && _vm.type === 'hour' ? _c('div', {
    ref: "inner",
    staticClass: "var-time-picker-clock__inner"
  }, _vm._l(_vm.hours24, function (hour, index) {
    return _c('div', {
      key: hour,
      staticClass: "var-time-picker-clock__item",
      class: [_vm.isActive(index, true) ? 'var-time-picker-clock__item--active' : null, _vm.isDisable(hour) ? 'var-time-picker-clock__item--disable' : null],
      style: _vm.getStyle(index, hour, true)
    }, [_vm._v("\n      " + _vm._s(hour) + "\n    ")]);
  }), 0) : _vm._e()], 2);
};

var staticRenderFns = [];

var _default = (0, _create.defineComponent)({
  render,
  staticRenderFns,
  name: 'Clock',
  props: {
    isInner: {
      type: Boolean,
      required: true
    },
    rad: {
      type: Number
    },
    format: {
      type: String,
      default: 'ampm'
    },
    allowedTime: {
      type: Object
    },
    time: {
      type: Object,
      required: true
    },
    useSeconds: {
      type: Boolean,
      default: false
    },
    preventNextUpdate: {
      type: Boolean,
      default: false
    },
    type: {
      type: String,
      default: 'hour'
    },
    ampm: {
      type: String,
      default: 'am'
    },
    color: {
      type: String
    },
    min: {
      type: String
    },
    max: {
      type: String
    }
  },
  data: () => ({
    inner: null,
    disableHour: [],
    disable24HourIndex: []
  }),
  computed: {
    hours24() {
      return _props.hours24;
    },

    handStyle() {
      return {
        transform: "rotate(" + (0, _shared.toNumber)(this.rad) + "deg)",
        height: this.isInner && this.type === 'hour' ? 'calc(50% - 40px)' : 'calc(50% - 4px)',
        backgroundColor: this.getHandleColor(),
        borderColor: this.getHandleColor()
      };
    },

    activeItemIndex() {
      if (this.rad === undefined) return;
      var value = this.rad / 30;
      return value >= 0 ? value : value + 12;
    },

    timeScales() {
      if (this.type === 'hour') return _props.hoursAmpm;
      return _props.minSec;
    },

    activeItemIndexAndIsInner() {
      return [this.activeItemIndex, this.isInner];
    }

  },
  methods: {
    isDisableMinSec(time, isDisable) {
      var _time;

      time = (_time = time) != null ? _time : this.type === 'minute' ? this.time.minute : this.time.second;
      var disableMethod = this.type === 'minute' ? _utils.getIsDisableMinute : _utils.getIsDisableSecond;
      var values = {
        time: (0, _shared.toNumber)(time),
        format: this.format,
        ampm: this.ampm,
        hour: this.time.hour,
        minute: (0, _shared.toNumber)(this.time.minute),
        max: this.max,
        min: this.min,
        allowedTime: this.allowedTime,
        disableHour: this.disableHour
      };
      if (isDisable && this.type === 'minute') Reflect.deleteProperty(values, 'minute');
      return disableMethod(values);
    },

    getHandleColor() {
      if (this.activeItemIndex === undefined) return this.color;
      var hour = this.isInner ? _props.hours24[this.activeItemIndex] : this.timeScales[this.activeItemIndex];

      if (this.timeScales === _props.minSec) {
        return this.isDisableMinSec() ? '#bdbdbd' : this.color;
      }

      return this.isDisable(hour) ? '#bdbdbd' : this.color;
    },

    isActive(index, inner) {
      if (inner) return this.activeItemIndex === index && this.isInner;
      return this.activeItemIndex === index && (!this.isInner || this.type !== 'hour');
    },

    isDisable(time) {
      if (this.type === 'hour') {
        if ((0, _utils.notConvert)(this.format, this.ampm)) return this.disableHour.includes(time);

        var timeIndex = _props.hoursAmpm.findIndex(hour => hour === time);

        return this.disable24HourIndex.includes(timeIndex);
      }

      return this.isDisableMinSec(time, true);
    },

    getStyle(index, hour, inner) {
      var rad = 2 * Math.PI / 12 * index - Math.PI / 2;
      var left = 50 * (1 + Math.cos(rad));
      var top = 50 * (1 + Math.sin(rad));

      var computedColor = () => {
        if (!this.isActive(index, inner)) {
          return {
            backgroundColor: undefined,
            color: undefined
          };
        }

        if (this.isDisable(hour)) {
          return {
            backgroundColor: '#bdbdbd',
            color: '#fff'
          };
        }

        return {
          backgroundColor: this.color,
          color: undefined
        };
      };

      var {
        backgroundColor,
        color
      } = computedColor();
      return {
        left: left + "%",
        top: top + "%",
        backgroundColor,
        color
      };
    },

    getSize() {
      var {
        width,
        height
      } = this.inner.getBoundingClientRect();
      return {
        width,
        height
      };
    },

    getHour() {
      if (this.activeItemIndex === undefined) return undefined;
      var hours = this.ampm === 'am' ? _props.hoursAmpm : _props.hours24;
      return hours[this.activeItemIndex].padStart(2, '0');
    }

  },
  watch: {
    activeItemIndexAndIsInner(_ref, _ref2) {
      var [index, inner] = _ref;
      var [oldIndex, oldInner] = _ref2;
      var isSame = index === oldIndex && inner === oldInner;
      if (isSame || this.type !== 'hour' || this.activeItemIndex === undefined) return;
      var newHour = inner ? _props.hours24[this.activeItemIndex] : this.getHour();
      var second = this.useSeconds ? ":" + this.time.second : '';
      var newTime = newHour + ":" + this.time.minute + second;
      if (!this.preventNextUpdate) this.$emit('update', newTime);
      this.$emit('change-prevent-update');
    },

    rad(rad, oldRad) {
      if (this.type === 'hour' || rad === undefined || oldRad === undefined) return;
      var radToMinSec = rad / 6 >= 0 ? rad / 6 : rad / 6 + 60;
      var oldRadToMinSec = oldRad / 6 >= 0 ? oldRad / 6 : oldRad / 6 + 60;
      if (radToMinSec === oldRadToMinSec) return;
      var newTime;
      var {
        hourStr
      } = (0, _utils.convertHour)(this.format, this.ampm, this.time.hour);

      if (this.type === 'minute') {
        var newMinute = (0, _dayjs.default)().minute(radToMinSec).format('mm');
        var second = this.useSeconds ? ":" + this.time.second : '';
        newTime = hourStr + ":" + newMinute + second;
      }

      if (this.type === 'second') {
        var newSecond = (0, _dayjs.default)().second(radToMinSec).format('ss');

        var _second = this.useSeconds ? ":" + newSecond : '';

        newTime = hourStr + ":" + this.time.minute + _second;
      }

      this.$emit('update', newTime);
    },

    max: {
      handler(newV) {
        var _this$allowedTime;

        this.disableHour = [];

        if (newV && !this.min) {
          var {
            hour: maxHour
          } = (0, _utils.getNumberTime)(newV);

          var disableAmpmHours = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          var disable24Hours = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          this.disableHour = [...disableAmpmHours, ...disable24Hours];
        }

        if (!newV && this.min) {
          var {
            hour: minHour
          } = (0, _utils.getNumberTime)(this.min);

          var _disableAmpmHours = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          var _disable24Hours = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          this.disableHour = [..._disableAmpmHours, ..._disable24Hours];
        }

        if (newV && this.min) {
          var {
            hour: _maxHour
          } = (0, _utils.getNumberTime)(newV);
          var {
            hour: _minHour
          } = (0, _utils.getNumberTime)(this.min);

          var _disableAmpmHours2 = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < _minHour || (0, _shared.toNumber)(hour) > _maxHour);

          var _disable24Hours2 = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < _minHour || (0, _shared.toNumber)(hour) > _maxHour);

          this.disableHour = [..._disableAmpmHours2, ..._disable24Hours2];
        }

        if ((_this$allowedTime = this.allowedTime) != null && _this$allowedTime.hours) {
          var {
            hours
          } = this.allowedTime;

          var _disableAmpmHours3 = _props.hoursAmpm.filter(hour => !hours((0, _shared.toNumber)(hour)));

          var _disable24Hours3 = _props.hours24.filter(hour => !hours((0, _shared.toNumber)(hour)));

          this.disableHour = [...new Set([...this.disableHour, ..._disableAmpmHours3, ..._disable24Hours3])];
        }

        this.disable24HourIndex = this.disableHour.map(hour => _props.hours24.findIndex(hour24 => hour === hour24)).filter(hour => hour >= 0);
      },

      immediate: true
    },
    min: {
      handler(newV) {
        var _this$allowedTime2;

        this.disableHour = [];

        if (this.max && !newV) {
          var {
            hour: maxHour
          } = (0, _utils.getNumberTime)(this.max);

          var disableAmpmHours = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          var disable24Hours = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          this.disableHour = [...disableAmpmHours, ...disable24Hours];
        }

        if (!this.max && newV) {
          var {
            hour: minHour
          } = (0, _utils.getNumberTime)(newV);

          var _disableAmpmHours4 = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          var _disable24Hours4 = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          this.disableHour = [..._disableAmpmHours4, ..._disable24Hours4];
        }

        if (this.max && newV) {
          var {
            hour: _maxHour2
          } = (0, _utils.getNumberTime)(this.max);
          var {
            hour: _minHour2
          } = (0, _utils.getNumberTime)(newV);

          var _disableAmpmHours5 = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < _minHour2 || (0, _shared.toNumber)(hour) > _maxHour2);

          var _disable24Hours5 = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < _minHour2 || (0, _shared.toNumber)(hour) > _maxHour2);

          this.disableHour = [..._disableAmpmHours5, ..._disable24Hours5];
        }

        if ((_this$allowedTime2 = this.allowedTime) != null && _this$allowedTime2.hours) {
          var {
            hours
          } = this.allowedTime;

          var _disableAmpmHours6 = _props.hoursAmpm.filter(hour => !hours((0, _shared.toNumber)(hour)));

          var _disable24Hours6 = _props.hours24.filter(hour => !hours((0, _shared.toNumber)(hour)));

          this.disableHour = [...new Set([...this.disableHour, ..._disableAmpmHours6, ..._disable24Hours6])];
        }

        this.disable24HourIndex = this.disableHour.map(hour => _props.hours24.findIndex(hour24 => hour === hour24)).filter(hour => hour >= 0);
      },

      immediate: true
    },
    allowedTime: {
      handler(newV) {
        this.disableHour = [];

        if (this.max && !this.min) {
          var {
            hour: maxHour
          } = (0, _utils.getNumberTime)(this.max);

          var disableAmpmHours = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          var disable24Hours = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) > maxHour);

          this.disableHour = [...disableAmpmHours, ...disable24Hours];
        }

        if (!this.max && this.min) {
          var {
            hour: minHour
          } = (0, _utils.getNumberTime)(this.min);

          var _disableAmpmHours7 = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          var _disable24Hours7 = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < minHour);

          this.disableHour = [..._disableAmpmHours7, ..._disable24Hours7];
        }

        if (this.max && this.min) {
          var {
            hour: _maxHour3
          } = (0, _utils.getNumberTime)(this.max);
          var {
            hour: _minHour3
          } = (0, _utils.getNumberTime)(this.min);

          var _disableAmpmHours8 = _props.hoursAmpm.filter(hour => (0, _shared.toNumber)(hour) < _minHour3 || (0, _shared.toNumber)(hour) > _maxHour3);

          var _disable24Hours8 = _props.hours24.filter(hour => (0, _shared.toNumber)(hour) < _minHour3 || (0, _shared.toNumber)(hour) > _maxHour3);

          this.disableHour = [..._disableAmpmHours8, ..._disable24Hours8];
        }

        if (newV != null && newV.hours) {
          var {
            hours
          } = newV;

          var _disableAmpmHours9 = _props.hoursAmpm.filter(hour => !hours((0, _shared.toNumber)(hour)));

          var _disable24Hours9 = _props.hours24.filter(hour => !hours((0, _shared.toNumber)(hour)));

          this.disableHour = [...new Set([...this.disableHour, ..._disableAmpmHours9, ..._disable24Hours9])];
        }

        this.disable24HourIndex = this.disableHour.map(hour => _props.hours24.findIndex(hour24 => hour === hour24)).filter(hour => hour >= 0);
      },

      immediate: true
    }
  },

  mounted() {
    this.inner = this.$refs.inner;
  }

});

exports.default = _default;