"use strict";

exports.__esModule = true;
exports.addRouteListener = addRouteListener;
exports.mountComponent = mountComponent;
exports.pickProps = pickProps;

var _vue = _interopRequireDefault(require("vue"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function pickProps(props, propsKey) {
  return Array.isArray(propsKey) ? propsKey.reduce((pickedProps, key) => {
    pickedProps[key] = props[key];
    return pickedProps;
  }, {}) : props[propsKey];
}

function mountComponent(component, container, options) {
  if (container === void 0) {
    container = 'body';
  }

  if (options === void 0) {
    options = {};
  }

  var instance = new (_vue.default.extend(component))(options);
  var el = instance.$mount().$el;
  var wrapper = document.querySelector(container);
  wrapper.appendChild(el);
  return {
    instance,

    unmount() {
      instance.$destroy();
      wrapper.removeChild(el);
    }

  };
}

function addRouteListener(vm, cb) {
  vm.$on('hook:mounted', () => {
    window.addEventListener('hashchange', cb);
    window.addEventListener('popstate', cb);
  });
  vm.$on('hook:beforeDestroy', () => {
    window.removeEventListener('hashchange', cb);
    window.removeEventListener('popstate', cb);
  });
}