"use strict";

exports.__esModule = true;
exports.createChildrenMixin = createChildrenMixin;
exports.createParentMixin = createParentMixin;
exports.sortChildren = sortChildren;

function flatVNodes(subTree) {
  var vNodes = [];

  function flat(subTree) {
    subTree.forEach(child => {
      vNodes.push(child);

      if (child.componentInstance) {
        flat(child.componentInstance.$children.map(item => item.$vnode));
      }

      if (child.children) {
        flat(child.children);
      }
    });
  }

  flat(subTree);
  return vNodes;
}

function sortChildren(children, parent) {
  var {
    componentOptions
  } = parent.$vnode;

  if (!componentOptions || !componentOptions.children) {
    return;
  }

  var vNodes = flatVNodes(componentOptions.children);
  children.sort((a, b) => vNodes.indexOf(a.$vnode) - vNodes.indexOf(b.$vnode));
}

function createParentMixin(parent, options) {
  if (options === void 0) {
    options = {};
  }

  var {
    childrenKey = 'children'
  } = options;
  return {
    provide() {
      return {
        [parent]: this
      };
    },

    data() {
      return {
        [childrenKey]: []
      };
    }

  };
}

function createChildrenMixin(parent, options) {
  if (options === void 0) {
    options = {};
  }

  var {
    indexKey = 'index',
    childrenKey = 'children',
    parentKey = 'parent'
  } = options;
  return {
    inject: {
      [parent]: {
        default: null
      }
    },
    computed: {
      [parentKey]() {
        return this[parent];
      },

      [indexKey]() {
        this["bind" + parentKey + childrenKey + "Relation"]();

        if (this[parentKey]) {
          return this[parentKey][childrenKey].indexOf(this);
        }

        return null;
      }

    },

    mounted() {
      this["bind" + parentKey + childrenKey + "Relation"]();
    },

    beforeDestroy() {
      if (this[parentKey]) {
        this[parentKey][childrenKey] = this[parentKey][childrenKey].filter(item => item !== this);
      }
    },

    methods: {
      ["bind" + parentKey + childrenKey + "Relation"]() {
        if (!this[parentKey] || this[parentKey][childrenKey].includes(this)) {
          return;
        }

        var children = [...this[parentKey][childrenKey], this];
        sortChildren(children, this[parentKey]);
        this[parentKey][childrenKey] = children;
      }

    }
  };
}