"use strict";

exports.__esModule = true;
exports.isURL = exports.isString = exports.isPlainObject = exports.isObject = exports.isNumber = exports.isHTMLSupportVideo = exports.isHTMLSupportImage = exports.isFunction = exports.isEmpty = exports.isBool = exports.isArray = exports.inBrowser = exports.easeInOutCubic = exports.dt = exports.delay = exports.debounce = exports.cubic = exports.createCache = exports.camelize = exports.bigCamelize = exports.NOOP = void 0;
exports.kebabCase = kebabCase;
exports.linear = void 0;
exports.parseFormat = parseFormat;
exports.uniq = exports.toNumber = exports.throttle = exports.removeItem = exports.range = void 0;

var cubic = value => Math.pow(value, 3);

exports.cubic = cubic;

var uniq = arr => [...new Set(arr)];

exports.uniq = uniq;

var dt = (value, defaultText) => value == null ? defaultText : value;

exports.dt = dt;

var linear = value => value;

exports.linear = linear;

var isHTMLSupportImage = val => {
  if (val == null) {
    return false;
  }

  return val.startsWith('data:image') || /\.(png|jpg|gif|jpeg|svg)$/.test(val);
};

exports.isHTMLSupportImage = isHTMLSupportImage;

var isHTMLSupportVideo = val => {
  if (val == null) {
    return false;
  }

  return val.startsWith('data:video') || /\.(mp4|webm|ogg)$/.test(val);
};

exports.isHTMLSupportVideo = isHTMLSupportVideo;

var easeInOutCubic = value => value < 0.5 ? cubic(value * 2) / 2 : 1 - cubic((1 - value) * 2) / 2;

exports.easeInOutCubic = easeInOutCubic;

var isString = val => typeof val === 'string';

exports.isString = isString;

var toNumber = val => {
  if (val == null) return 0;

  if (isString(val)) {
    val = parseFloat(val);
    val = Number.isNaN(val) ? 0 : val;
    return val;
  }

  if (isBool(val)) return Number(val);
  return val;
};

exports.toNumber = toNumber;

var isBool = val => typeof val === 'boolean';

exports.isBool = isBool;

var isFunction = val => typeof val === 'function';

exports.isFunction = isFunction;

var isNumber = val => typeof val === 'number';

exports.isNumber = isNumber;

var isPlainObject = val => Object.prototype.toString.call(val) === '[object Object]';

exports.isPlainObject = isPlainObject;

var isObject = val => typeof val === 'object' && val !== null;

exports.isObject = isObject;

var isArray = val => Array.isArray(val);

exports.isArray = isArray;

var isURL = val => {
  if (!val) {
    return false;
  }

  return /^(http)|(\.*\/)/.test(val);
};

exports.isURL = isURL;

var isEmpty = val => val === undefined || val === null || val === '' || Array.isArray(val) && !val.length;

exports.isEmpty = isEmpty;

var removeItem = (arr, item) => {
  if (arr.length) {
    var index = arr.indexOf(item);

    if (index > -1) {
      return arr.splice(index, 1);
    }
  }
};

exports.removeItem = removeItem;

var throttle = function (method, mustRunDelay) {
  if (mustRunDelay === void 0) {
    mustRunDelay = 200;
  }

  var timer;
  var start = 0;
  return function loop() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var now = Date.now();
    var elapsed = now - start;

    if (!start) {
      start = now;
    }

    if (timer) {
      window.clearTimeout(timer);
    }

    if (elapsed >= mustRunDelay) {
      method.apply(this, args);
      start = now;
    } else {
      timer = window.setTimeout(() => {
        loop.apply(this, args);
      }, mustRunDelay - elapsed);
    }
  };
};

exports.throttle = throttle;

var debounce = function (method, delay) {
  if (delay === void 0) {
    delay = 200;
  }

  var timer;
  return function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    timer && window.clearTimeout(timer);
    timer = window.setTimeout(() => {
      method.apply(this, args);
    }, delay);
  };
};

exports.debounce = debounce;

function kebabCase(str) {
  var reg = /([^-])([A-Z])/g;
  return str.replace(reg, '$1-$2').replace(reg, '$1-$2').toLowerCase();
}

var camelize = s => s.replace(/-(\w)/g, (_, p) => p.toUpperCase());

exports.camelize = camelize;

var bigCamelize = s => camelize(s).replace(s.charAt(0), s.charAt(0).toUpperCase());

exports.bigCamelize = bigCamelize;

var range = (num, left, right) => {
  if (num < left) {
    return left;
  }

  if (num > right) {
    return right;
  }

  return num;
};

exports.range = range;

function parseFormat(format, time) {
  var scannedTimes = Object.values(time);
  var scannedFormats = ['DD', 'HH', 'mm', 'ss'];
  var padValues = [24, 60, 60, 1000];
  scannedFormats.forEach((scannedFormat, index) => {
    if (!format.includes(scannedFormat)) {
      scannedTimes[index + 1] += scannedTimes[index] * padValues[index];
    } else {
      format = format.replace(scannedFormat, String(scannedTimes[index]).padStart(2, '0'));
    }
  });

  if (format.includes('S')) {
    var ms = String(scannedTimes[scannedTimes.length - 1]).padStart(3, '0');

    if (format.includes('SSS')) {
      format = format.replace('SSS', ms);
    } else if (format.includes('SS')) {
      format = format.replace('SS', ms.slice(0, 2));
    } else {
      format = format.replace('S', ms.slice(0, 1));
    }
  }

  return format;
}

var delay = time => new Promise(resolve => setTimeout(resolve, time));

exports.delay = delay;

var inBrowser = () => typeof window !== 'undefined';

exports.inBrowser = inBrowser;

var createCache = max => {
  var cache = [];
  return {
    cache,

    has(key) {
      return this.cache.includes(key);
    },

    add(key) {
      if (this.has(key)) {
        return;
      }

      this.cache.length === max && cache.shift();
      this.cache.push(key);
    },

    remove(key) {
      this.has(key) && removeItem(this.cache, key);
    },

    clear() {
      this.cache.length = 0;
    }

  };
};

exports.createCache = createCache;

var NOOP = () => {};

exports.NOOP = NOOP;